//
// globals
//

var storagePrefix = 'vlog_';

var vid      = null;

var CODES = null;
var READOUT = null;
var TRANSCRIPT = null;
var HOPS = null;
var BLANK = null;


//
// a debug system, from Flanagan's book
//


var debug = function (msg) {
    console.log (msg);
}


//
// tape recorder ops
//


var playStop = function () {
    try {
	// if (vid.ended) vid.currentTime = 0;
	if (vid.paused) vid.play(); else vid.pause();
    }
    catch (err) {
	// no op
    }
}


var speed = {
    current : 1.00,

    speedList : [ 0.10, 0.20, 0.50, 1.00, 2.00 ],

    change : function () {

	var j = this.speedList.indexOf (this.current);

	if (j < 0) {
	    // not found
	    this.current = 1.00;
	}
	else {
	    j += 1;
	    j %= this.speedList.length;
	    this.current = this.speedList[j];
	}
	vid.defaultPlaybackRate = this.current;
	vid.playbackRate = this.current;

    },
}


var tapeSymbols = {
    'pause'  : '&#10073;&nbsp;&#10073;',
    'play'   : '&#9654;',
    'back'   : '&#9664;',
    'ff'     : '&#9654;&#9654;',
    'rewind' : '&#9664;&#9664;',
    'stop'   : '&#9724;',
    'eject'  : '&#9167;',
    'up'     : '&#9650;',
    'down'   : '&#9660;',
    'dot'    : '&#9679;',
}


//
// various readouts need to be kept current
//


var updateLegends = function () {
    var t;

    t = document.getElementById ('playStopBtn');
    if (vid.paused) t.innerHTML = tapeSymbols ['play'];
    else            t.innerHTML = tapeSymbols ['pause'];

    t = document.getElementById ('speedChange');
    t.innerHTML = speed.current.toFixed (2);

    t = document.getElementById ('hopL');
    t.innerHTML =  tapeSymbols.back + ' ' + hop.current;

    t = document.getElementById ('hopR');
    t.innerHTML = hop.current + ' ' + tapeSymbols.play;

    t = document.getElementById ('clock');
    t.innerHTML = vid.currentTime.toFixed (2);

    t = document.getElementById ('mark');
    t.innerHTML = hop.mark.toFixed (2);

    t = document.getElementById ('excursion');
    t.innerHTML = (vid.currentTime - hop.mark).toFixed (2);

    t = document.getElementById ('editWhich');
    if (t) t.innerHTML = codeEditor.editWhich;
}


//
// text selection for copy assistance
//


var Selection = function (element) {
    var e = document.getElementById (element);
    if (e) {
	// lots of answers for mozilla and IE.
	// webkit is a mystery.
	// See rangy at google code.

	var R = document.createRange();
	R.selectNodeContents(e);

	var S = window.getSelection();
	S.removeAllRanges();
	S.addRange(R);
    }
    return;
}


//
// shuffling z-levels
//


var zList  = {
    'help'   : 1,
    'report' : 2,
    'wrench' : 3,
    'perimeter' : 10
};


var defaultZ = function () {
    var j;
    for (j in zList) {
	document.getElementById (j).style.zIndex = zList[j];
    }
    saveTranscript ();
}


var raiseDiv = function (j) {
    defaultZ();
    document.getElementById (j).style.zIndex = 99;
}


var reportMode = function () {
//    document.getElementById ('transcript').style.height = '1px';
// (	(window.innerHeight - 92).toFixed(0) + 'px'    );

    return raiseDiv ('report');
 }

var helpMode   = function () {
    return raiseDiv ('help');
}


var wrenchMode = function () {
    //wrenchText[wrenchPage]
    return raiseDiv ('wrench');
}

var helpInNewWindow = function () {
    window.open ('help.html',
		 '_blank',
		 ''
		);
    return;
}


//
// video
//


var videoName = function (msg, useclass) {
    var node = document.getElementById ('vTitle');
    node.innerHTML = msg;
    node.classname = useclass;
}


var canPlayHandler = function(evt) {
    // once the movie is [partly] loaded,
    // can make geometry decisions

    var vidVertOverhead = function () {
	var h = vid.height;
	if (h < 1) h = vid.videoHeight;
	// that is, displayed height if specified else natural height

	return (document.getElementById ('movie').clientHeight
		-
		h
	);
    }

    var goodVidHeight = function () {
	// for good arithmetic, with absolute positions,
	// set zero margin and padding on body.
	var a = (window.innerHeight
		 -
		 document.getElementById ('sideBySide').clientHeight
		 -
		 vidVertOverhead ()
		 -
		 document.getElementById ('vTitle').clientHeight
		 -
		 document.getElementById ('sig').clientHeight
		);
	var b = vid.videoHeight * 4;

	return Math.max (160, Math.min (a, b));
    }

    // make the video as large as will fit.
    vid.height= goodVidHeight();
}


//
// Dimitar Bonev
// http://stackoverflow.com/questions/8885701/
//        play-local-hard-drive-video-file-with-html5-video-tag
// http://jsfiddle.net/cCCZ2/
//
// which soes not work in Safari 6 --
// html5 video does not mix with blob URLs
//


function localFileVideoPlayerInit (win) {
    var URL = win.URL || win.webkitURL;

    var playSelectedFile = function playSelectedFileInit(event) {
        var file = this.files[0];
        var type = file.type;
        var videoNode = document.querySelector('video');
        var canPlay = videoNode.canPlayType(type);

        canPlay = (canPlay === '' ? 'no' : canPlay);

	if (canPlay === 'no') {
	    videoName ('Cannot play type ' + type, 'red');
	    return;
	}

        var fileURL = URL.createObjectURL(file);

        videoNode.src = fileURL;
        videoName (file.name, '');
	makeNoteInRed (file.name);
    };

    var inputNode = document.getElementById ('frontFileBtn')

    if (!URL) {
        alert  ('Your browser does not handle blob URLs.');
        return;
    }

    inputNode.addEventListener('change', playSelectedFile, false);
}


//
// Robert Gravelle
// http://www.htmlgoodies.com/print/HTML5/javascript/
//        drag-files-into-the-browser-from-the-desktop-HTML5.html
// http://jsfiddle.net/JtJ5N/1/
//


function addEventHandler(obj, evt, handler) {
    if (obj.addEventListener) {
        // W3C method
        obj.addEventListener (evt, handler, false);
    }
    else if (obj.attachEvent) {
        // IE method.
        obj.attachEvent ('on'+evt, handler);
    }
    else {
        // Old school method.
        obj['on'+evt] = handler;
    }
}

var dfile = null;

if (true) {
    var drop;

    addEventHandler (window, 'load', function() {
	drop = document.getElementById ('movie');

	function cancel(e) {
	    if (e.preventDefault) { e.preventDefault(); }
	    return false;
	}

	// Tells the browser that we *can* drop on this target
	addEventHandler (drop, 'dragover', cancel);
	addEventHandler (drop, 'dragenter', cancel);

	addEventHandler (drop, 'drop', function (e) {
	    e = e || window.event;
	    // get window.event if e argument missing (in IE)
	    if (e.preventDefault) { e.preventDefault(); }
	    // stops the browser from redirecting off to the image.

	    var URL = window.URL || window.webkitURL;

	    var files = e.dataTransfer.files;

	    for (var i=0; i<files.length; i++) {
		var file = files[i];

		// dfile = file; // debug

		var videoNode = document.querySelector ('video');
		var playable  = videoNode.canPlayType (file.type);

		if ((playable === '') || (playable === 'no')) {
		    console.log ('not playable');
		    return;
		}

		var fileURL = URL.createObjectURL (file);
		videoNode.src = fileURL;
		videoName (file.name, 'orange')
		document.f1.reset();
		makeNoteInRed (file.name);

	    }
	    return false;
	});

    });
}

else {
    alert ('missing feature: HTML5 FileReader');
}


//
// event recording
//



var saveTranscript = function () {
    var k = storagePrefix + 'transcript';
    localStorage[k] = TRANSCRIPT.innerHTML;
}


var logText = function (text) {
    var movieTime = vid.currentTime;
    var now = new Date().getTime();

    var a = [];
    a.push (movieTime.toFixed(2));
    a.push (text);

    var t1 = vsprintf ('%8.8s  %s', a);
    READOUT.innerHTML = (t1);

    a = [];
    a.push (movieTime.toFixed(2));
    a.push (now);
    a.push (text);

    var t2;

    var t2 = document.createElement('div');
    t2.innerHTML  = vsprintf ('%8.8s  %s  %s', a);
    TRANSCRIPT.appendChild (t2);
}


var logTextFromCodeBox = function (text) {
    logText (text);

    // if ckick on a code button, the button will (IE10, firefox)
    // take focus from its parent box, or not (chrome).
    // This fixes that, but at the price of a visual flash.
    // CODES.focus ();
}


var logKey = function (evt) {
    var k = KeyCode.translate_event (evt);
    var kname = KeyCode.hot_key (k);

    var text = '';

    // if (k.alt  ) text = text +   'alt-';
    // if (k.ctrl ) text = text +  'ctrl-';
    // if (k.shift) text = text + 'shift-';
    // text += kname;

    text = kname;

    // var k = KeyCode.hot_key (KeyCode.translate_event (evt));

    // a few special keys
    if (kname === 'Space') {
	if (vid.paused) {
	    vid.play();
	    logText ('play_video')
	}
	else {
	    vid.pause();
	    logText ('pause_video')
	}
	saveTranscript ();
    }

    // if 'Tab' then blur?
    else if (kname === 'Tab') {
	evt.target.blur();
    }

    // don't log these --  Shift Ctrl Alt
    else if (kname === 'Shift') { ; }
    else if (kname ===  'Ctrl') { ; }
    else if (kname ===   'Alt') { ; }

    // and finally
    else {
	logText (text);
//	logText ('code = ' + k.code);
	saveTranscript ();
    }
}


var logBtn = function (evt) {
    var k = evt.target.innerText.trim();
    logText (k);
    saveTranscript ();
}


var makeNoteInRed = function (text) {

    var now = new Date().toLocaleString();

    var a = ''
    a += '<font color=red>\n'
    a += '## '
    a += now
    a += '\n'
    a += '## '
    a += text
    a += '\n\n'
    a += '</font>\n'

    var t2 = document.createElement('div');
    t2.innerHTML  = a;
    TRANSCRIPT.appendChild (t2);
    saveTranscript ();
}


//
// loggable words
//


var builtinWords = new Array (
    "",

    "A B C D\nE F G H\nI J K L\nM N O P\nQ R S T\nU V W X\nY Z",

    ".b 7 8 9 /\n4 5 6 x\n1 2 3 -\n0 . % +\na b c d",

    ".T Lorem ipsum dolor sit amet consectetur adipisicing elit",

    ".b sed do eiusmod tempor incididunt ut labore et dolore magna aliqua\n\
.T Pigs may fly but they make unlikely birds .r .t \
Here lies poor Fred \n who was alive and is dead, \n\
Had it been his father \n I had much rather, \n\
Had it been his sister \n nobody would have missed her, \n\
Had it been his brother, \n still better than another, \n\
Had it been the whole generation, \n so much better for the nation, \n\
But since it is Fred \n who was alive and is dead, \n\
There is no more to be said. \
"

);


var ActiveWords = new Array ();
var useTheseWords = 1;


var wordGen = function (rawText) {

    // alert ('wordGen(): ' + rawText);

    var wordMode = "cb";
    // ct, cb, or tx, for clickable text, clickable button, dead text

    var text = "";

    var lines = rawText.split (/\n/)
    for (var L in lines) {
	//sline = line.trim;
	var words = lines[L].split (/ /);

	// (in passing) google chrome may choke here
	for (var W in words) {
	    if (words[W].length) {

		if      (words[W] == ".T") wordMode = "ct";
		else if (words[W] == ".t") wordMode = "tx";
		else if (words[W] == ".B") wordMode = "cb";
		else if (words[W] == ".b") wordMode = "cb";

		else if (words[W] == ".r") text += "<hr class=minor>\n";
		else if (words[W] == ".R") text += "<hr class=minor>\n";

		else if (wordMode == "ct") {
		    // span-ify
		    text += '<span class="live" role="button"'
		    text += ' onclick="logText (\'';
		    text += words[W];
		    text += '\');">';

		    text += words[W];
		    text += '</span>\n';
		}

		else if (wordMode == "cb") {
		    // button-ify
		    text += '<button class="varBtn" ';
		    text += ' onclick="logTextFromCodeBox (\'';
		    text += words[W];
		    text += '\');">';

		    text += words[W];
		    text += '</button>\n';
		}

		else if (wordMode == "tx") {
		    // span-ify
		    text += '<span class="dead">'
		    text += words[W];
		    text += '</span>\n';
		}

	    } // end if (words[W].length)
	} // end for (words)
	text += '<br>\n';
    } // end for (L in lines)
    return text;
}


var loadWords = function () {
    var j;
    var text = "";

    // provisionally ...
    for (j=0; j<builtinWords.length; j++) {
	ActiveWords[j] = builtinWords[j];
    }

    // stored over-rides
    for (j=1; j<builtinWords.length; j++) {

	key = storagePrefix + "words" + j
	if (key in localStorage) {
	    text = localStorage.getItem(key);
	    text = text.trim();
	    if (text.length)
		ActiveWords[j] = text;
	    else
		ActiveWords[j] = builtinWords[j];
	}
    }
}


var showCurrentWords = function () {
    W = document.getElementById ('wordSetBtn');
    W.innerHTML = useTheseWords.toString();

    W = document.getElementById ('codes');
    W.innerHTML = wordGen (ActiveWords[useTheseWords]);
}


var nextWordSet = function () {

    useTheseWords += 1;
    if (useTheseWords >= builtinWords.length)
	useTheseWords = 1; // not zero

    showCurrentWords ();
}


var firstWordSet = function () {
    useTheseWords = 0;
    nextWordSet ();
}


//
// storage
//


var eraseLike = function (prefix) {

    var key;
    var lead = storagePrefix + prefix;
    var head;

    for (key in localStorage) {
	head = key.substring (0, lead.length);
	if (head == lead) localStorage.removeItem (key);
    }
}


var eraseTranscript = function () {
    if (confirm ("Erase the transcript?")) {
	eraseLike ('transcript');
	TRANSCRIPT.innerHTML = '';
    }
}


var eraseWords = function () {
    if (confirm ("Undo changes to code lists?")) {
	eraseLike ('words');
	showCurrentWords();
    }
}


var eraseEverything = function () {
    if (confirm ("Erase all data and settings?")) {
	if (confirm ("REALLY erase all data and settings?")) {

	    // not this -- we're not the only web app on site
	    // localStorage.clear();

	    eraseTranscript();
	    eraseWords ()
	    eraseLike (''); // anything 'spurious'
	}
    }
}


//
// wrench!
//


var wrenchPage = 0;

var nextWrench = function () {
    wrenchPage += 1;
    if (wrenchPage  >= wrenchText.length)
	wrenchPage  = 1; // not zero

    showCurrentWrench ();
}


var firstWrench = function () {
    wrenchPage = 0;
    nextWrench ();
}


var showCurrentWrench = function () {
    var d = document.getElementById ('currentTool');
    d.innerHTML = wrenchText[wrenchPage];
};


var codeEditor = {

    editWhich : 1,

    editNext : function () {
	this.editWhich += 1;
	if (this.editWhich >=  builtinWords.length)
	    this.editWhich = 1; // not zero
    },


    loadForEdit : function () {
	var d = document.getElementById ('codeEditBox');
	d.innerHTML = ActiveWords[this.editWhich];
    },

    saveEdit : function () {
	var d = document.getElementById ('codeEditBox');
	var newVersion = d.innerText.trim();
	var key = storagePrefix + 'words' + this.editWhich;

	// alert ('new version>'+newVersion+'<');

	if (newVersion.length) {
            ActiveWords[this.editWhich] = newVersion;
            localStorage.setItem (key, newVersion);
        }
        else {
            ActiveWords[this.editWhich] = builtinWords[this.editWhich];
            localStorage.removeItem (key);
        }

	showCurrentWords();

    },

};


var wrenchText = [
    '',

    '\
<h2>code editor</h2>\
\
<table class="narrowRight">\
<tr>\
<td>\
<div id="codeEditBox" class="tt" contenteditable="true"></div>\
<td>\
<button id="editWhich"\
        onclick="codeEditor.editNext();">&nbsp;</button>\
<br>\
<button onclick="codeEditor.loadForEdit();">load</button>\
<br>\
<button onclick="codeEditor.saveEdit();">save</button>\
</table>\
\n\
\
<h2>storage</h2>\
\
<table class="narrowRight">\
<tr>\
<td><button style="width:100%;" onclick="eraseTranscript();">\
erase the transcript</button>\
<tr>\
<td><button style="width:100%;" onclick="eraseWords();">\
undo changes to code lists</button>\
</table>\
\n\
',


];


//
// keyboard setup
//


var liveBoxSetup = function () {

    CODES.onkeydown = function (e) {
	e = e || window.event
	e.preventDefault();
	logKey (e);
	return false;
    };

    CODES.onkeypress = function (e) {
	e.preventDefault();
	return false;
    };

    CODES.onkeyup = function (e) {
	// KeyCode.key_up(e);
	e.preventDefault();
	return false;
    }

}


var isBlankComplete = function (event) {
    if (event.keyCode == 13) {
	try { vid.currentTime = event.target.value; }
	catch (err) { ; }
	event.preventDefault();
    }
}


var blankLoosesFocus = function (event) {
    // vid.currentTime = event.target.value; // probably not convenient
}


var hop = {
    mark : 0.00,

    steps : [ "2", "1", "0.5", "0.1" ],

    current : "2",

    coarser : function () {
	var j = this.steps.indexOf (this.current);

	j -= 1;
	if (j < 0) j = 0;
	this.current = this.steps[j];
    },

    finer : function () {
	var j = this.steps.indexOf (this.current);
	var last = this.steps.length - 1;

	if (j < 0) j = last;		// not found
	j += 1;
	if (j > last) j = last;
	this.current = this.steps[j];
    },

    // if no movie loaded, then duration is NaN

    left : function () {
	if (!vid.paused) vid.pause();

	var here = vid.currentTime;
	here -= Number (this.current);
	try { vid.currentTime = here; }
	catch (err) { ; }
    },

    right : function () {
	if (!vid.paused) vid.pause();

	var here = vid.currentTime;
	here += Number (this.current);
	try { vid.currentTime = here; }
	catch (err) { ; }
    },

    toMark : function () {
	if (!vid.paused) vid.pause();
	try { vid.currentTime = this.mark; }
	catch (err) { ; }
    },

    setMarkFromEntry : function () {
	this.mark = BLANK.value;
    },

    setMarkFromMovie : function () {
	if (!vid.paused) vid.pause();
	this.mark = vid.currentTime;
    },

};


var hopProcessKey = function (evt) {
    var k = KeyCode.hot_key (KeyCode.translate_event (evt));

    if (k == 'Up') hop.coarser();
    else if (k == 'Down') hop.finer();
    else if (k == 'Left') hop.left();
    else if (k == 'Right') hop.right();
    else if (k == '>') hop.toMark();

    else if (k == 'Tab') {
	evt.target.blur();
    }

    else {
	; // no op
    }
}


var hopsSetup = function () {

    HOPS.onkeydown = function (e) {
	e = e || window.event
	e.preventDefault();
	hopProcessKey (e);
	return false;
    };

    HOPS.onkeypress = function (e) {
	e.preventDefault();
	return false;
    };

    HOPS.onkeyup = function (e) {
	// KeyCode.key_up(e);
	e.preventDefault();
	return false;
    }

}


//
// http://stackoverflow.com/questions/982214/
// how-do-a-get-buttons-not-to-take-the-focus
//
// a button should refuse focus if following
// is connected to onmousedown
//
// necessary -- Firefox 23 / win 7 (but not mac!)
//
// DOES MOT WORK -- firefox 23 / win 7 -- a.e., aug 23 2013
// HELPS         -- ie10               -- a.e., aug 23 2013
//

var noFocus = function () {
    var ae = document.activeElement;
    setTimeout (function() { ae.focus() }, 1);
}

var pFocus = function (el) {
    el.parentNode.focus();
}

//
// some globals
//


var ld = function () {

    localFileVideoPlayerInit (window);

    vid = document.getElementById ('vidclip');
    CODES   = document.getElementById ('codes');
    READOUT = document.getElementById ('readout');
    TRANSCRIPT = document.getElementById ('transcript');
    BLANK = document.getElementById ('dataEntry');
    HOPS = document.getElementById ('hops');

    // editing event listeners.
    TRANSCRIPT.addEventListener ('blur',
				 (
				     function () {
					 // this.designMode = 'off';
					 this.contenteditable = 'false';
					 // saveTranscript ();
				     }
				 ),
				 false
				);
    // NOTE TO SELF -- there's a blur event for every keypress.
    // That has to be wrong -- I hope
    // Meanwhile saveTranscript (); above turned off
    // Aug 18 2011 -- Safari 5.1 on OSX.

    TRANSCRIPT.addEventListener ('focus',
				 (
				     function () {
					 // works for chrome, fails for ie10
					 // this.designMode = 'on';
					 // works, both chrome and ie10
					 this.contenteditable = 'true';
					 // debug ('transcript focus');
				     }
				 ),
				 false
				);

    TRANSCRIPT.addEventListener ('change',
				 (
				     function () {
					 debug ('transcript change');
				     }
				 ),
				 false
				);
    // that seems not to happen

    // preserve the transcript

    var k = storagePrefix + 'transcript';
    if (k in localStorage) TRANSCRIPT.innerHTML = localStorage[k];
    else localStorage[k] = 'enpty';

    window.addEventListener ('beforeunload', saveTranscript, false);



    window.onresize = function (evt) {
	canPlayHandler();
    }

    // TRANSCRIPT.style.height = '1px';	// firefox, ie10 fixup

    loadWords ();
    firstWordSet ();

    firstWrench ();
    // codeEditor.loadForEdit();


    // keystroke handlers for the event logging div
    liveBoxSetup ();

    // keystroke handlers for the arrow key shortcuts
    hopsSetup  ();

    // update various readouts and button legends
    window.setInterval (updateLegends, 50);
}
